from rest_framework import generics, views, status, permissions
from rest_framework.response import Response

from commerce.models.product import Product
from commerce.serializers.product_serializer import ProductSerializer, GenericProductSerializer
from commerce.utils.response import prepare_create_success_response, prepare_success_response, prepare_error_response


class AllProductsView(generics.ListAPIView):
    queryset = Product.objects.all()
    serializer_class = GenericProductSerializer


class CreateListProductView(generics.ListCreateAPIView):
    queryset = Product.objects.all()
    serializer_class = ProductSerializer
    permission_classes = (permissions.IsAuthenticated,)

    def post(self, request, *args, **kwargs):
        try:
            serializer = ProductSerializer(data=request.data)
            if serializer.is_valid(raise_exception=True):
                serializer.save(owner=self.request.user)
                return Response(prepare_create_success_response(serializer.data), status=status.HTTP_201_CREATED)
            else:
                return Response(prepare_error_response('You have no permission to add product'),
                                status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            return Response(prepare_success_response(str(e)), status=status.HTTP_400_BAD_REQUEST)

    def list(self, request, *args, **kwargs):
        product = Product.objects.filter(owner=self.request.user)
        if product:
            serializer = ProductSerializer(product, many=True)
            return Response(prepare_success_response(serializer.data), status=status.HTTP_200_OK)
        else:
            return Response(prepare_error_response('No Product found'), status=status.HTTP_400_BAD_REQUEST)


class ProductDetailsUpdateView(generics.RetrieveUpdateAPIView):
    queryset = Product.objects.all()
    serializer_class = ProductSerializer
    lookup_field = 'slug'

    def update(self, request, *args, **kwargs):
        _slug = self.request.data.get('slug')
        serializer = ProductSerializer(Product.objects.get(slug=_slug), data=request.data, partial=True)
        if serializer.is_valid(raise_exception=True):
            serializer.save(owner=request.user)
            return Response(prepare_create_success_response(serializer.data), status=status.HTTP_201_CREATED)
        return Response(prepare_error_response(serializer.errors), status=status.HTTP_400_BAD_REQUEST)


class ProductDeleteAPIView(views.APIView):

    def get_object(self, pk):
        try:
            return Product.objects.get(id=pk)
        except Product.DoesNotExist:
            return None

    def delete(self, request, pk):
        instance = self.get_object(pk)
        if instance:
            instance.delete()
            return Response(prepare_success_response('Data deleted successfully'), status=status.HTTP_204_NO_CONTENT)
        else:
            return Response(prepare_error_response('Content Not found'), status=status.HTTP_400_BAD_REQUEST)



import django_filters
from commerce.models.product import Product


# Product Filter
class ProductFilter(django_filters.FilterSet):
    class Meta:
        model = Product
        fields = ['price', 'is_available']



from django.db import models
from django.contrib.auth.models import User
from django.utils.text import slugify
from django.dispatch import receiver
from django.db.models.signals import pre_save

from commerce.models import Category, Tag, Model, Brand
from commerce.models.core import CoreEntity
from commerce.utils.enum import TYPES


class Product(CoreEntity):
    item_name = models.CharField(max_length=120)
    slug = models.SlugField(unique=True, blank=True, null=True)
    owner = models.ForeignKey(User, on_delete=models.CASCADE, related_name='item_shop')
    categories = models.ForeignKey(Category, on_delete=models.CASCADE, related_name='item_category')
    brand_name = models.ForeignKey(Brand, on_delete=models.CASCADE, related_name='item_brand', blank=True, null=True)
    model_number = models.ForeignKey(Model, on_delete=models.CASCADE, related_name='item_model', blank=True, null=True)
    tags = models.ManyToManyField(Tag, related_name='item_tag', blank=True)
    is_available = models.BooleanField(default=True)
    price = models.DecimalField(max_digits=10, decimal_places=7)
    discount_price = models.DecimalField(max_digits=10, decimal_places=7)
    short_description = models.TextField(default='')
    serial_number = models.CharField(max_length=90, blank=True, null=True)
    item_type = models.IntegerField(choices=TYPES.select_types(), default=TYPES.KG.value)
    item_image = models.ImageField(upload_to='item/%y/%m', blank=True, null=True)
    galley_image = models.ImageField(upload_to='item/gallery/%y/%m', blank=True, null=True)
    galley_image2 = models.ImageField(upload_to='item/gallery/%y/%m', blank=True, null=True)
    galley_image3 = models.ImageField(upload_to='item/gallery/%y/%m', blank=True, null=True)

    def __str__(self):
        return self.item_name[:50]

    def save(self, *args, **kwargs):
        value = self.item_name
        self.slug = slugify(value, allow_unicode=True)
        super().save(*args, **kwargs)


@receiver(pre_save, sender=Product)
def store_pre_save(sender, instance, *args, **kwargs):
    if not instance.slug:
        instance.slug = slugify(instance.item_name)



import uuid

from django.db import models
from django.contrib.auth.models import User

from commerce.models import Product
from commerce.models.core import CoreEntity
from commerce.utils.enum import STATUS, PAYMENT


class OrderItem(CoreEntity):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='customer')
    ordered = models.BooleanField(default=False)
    item = models.ForeignKey(Product, on_delete=models.CASCADE)
    quantity = models.IntegerField(default=1)

    def __str__(self):
        return f"Item: {self.item} Quantity: {self.quantity}"

    def get_total_price(self):
        return self.quantity * self.item.price

    def get_total_discounted_price(self):
        return self.quantity * self.item.discount_price

    def get_amount_saved(self):
        return self.get_total_price() - self.get_total_discounted_price()

    def get_final_price(self):
        if self.item.discount_price:
            return self.get_total_discounted_price()
        return self.get_total_price()


class Order(CoreEntity):
    customer = models.ForeignKey(User, on_delete=models.CASCADE, related_name='orderCustomer')
    ref_code = models.CharField(max_length=20, default="0")
    items = models.ManyToManyField(OrderItem, related_name='orderItem')
    ordered = models.BooleanField(default=False)
    billing_address = models.CharField(max_length=150, blank=True, null=True)
    being_delivered = models.BooleanField(default=False)
    received = models.BooleanField(default=False)
    order_ide = models.CharField(max_length=25, blank=True, null=True, unique=True)
    status = models.IntegerField(choices=STATUS.order_status(), default=STATUS.PENDING.value)
    payment = models.IntegerField(choices=PAYMENT.payment_choices(), default=PAYMENT.CASH_ON_DELIVERY.value)

    def __str__(self):
        return f"OrderID: {self.order_ide} || Orders: {self.items.all()}"

    def get_total(self):
        total = 0
        for order_item in self.items.all():
            total += order_item.get_final_price()
        return total

    def save(self, *args, **kwargs):
        system_code = self.order_ide
        if not system_code:
            system_code = uuid.uuid4().hex[:6].upper()
        while Order.objects.filter(order_ide=system_code).exclude(pk=self.pk).exists():
            system_code = uuid.uuid4().hex[:6].upper()
        self.order_ide = system_code
        super(Order, self).save(*args, **kwargs)



from django.contrib import admin
from commerce.models.category import Category, Tag, Model, Brand
from commerce.models.product import Product
from commerce.models.order import Order, OrderItem

admin.site.register(Category)
admin.site.register(Tag)
admin.site.register(Model)
admin.site.register(Brand)
admin.site.register(Product)
admin.site.register(Order)
admin.site.register(OrderItem)



from commerce.models.category import Category, Tag, Model, Brand
from commerce.models.product import Product
from commerce.models.order import Order, OrderItem

__author = 'Sagor'

__all__ = [
    'Category',
    'Tag',
    'Model',
    'Brand',
    'Product',
    'Order',
    'OrderItem',
]



def prepare_create_success_response(serializer_data):
    """ prepare success response for all serializer """
    response = {
        'type': 'success',
        'message': 'Data Successfully created',
        'data': serializer_data
    }
    return response


def prepare_success_response(serializer_data):
    """ prepare success response for all serializer """
    response = {
        'type': 'success',
        'message': 'Data successfully returned',
        'data': serializer_data
    }
    return response


def prepare_error_response(serializer_error):
    """ prepare error response for all serializer """
    response = {
        'type': 'error',
        'message': serializer_error,
        "data": None
    }
    return response



from rest_framework import generics, views, status, permissions
from rest_framework.response import Response

from commerce.models.order import Order, OrderItem
from commerce.serializers.order_serializer import OrderSerializer, OrderItemSerializer, OrderListDetailSerializer
from commerce.utils.response import prepare_create_success_response, prepare_success_response, prepare_error_response


class CreateListOrderItemView(generics.ListCreateAPIView):
    queryset = OrderItem.objects.filter(ordered=False)
    serializer_class = OrderItemSerializer

    def post(self, request, *args, **kwargs):
        try:
            serializer = OrderItemSerializer(data=request.data)
            if serializer.is_valid(raise_exception=True):
                serializer.save(user=self.request.user)
                return Response(prepare_create_success_response(serializer.data), status=status.HTTP_201_CREATED)
            return Response(prepare_error_response(serializer.errors), status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            return Response(prepare_success_response(str(e)), status=status.HTTP_400_BAD_REQUEST)

    def list(self, request, *args, **kwargs):
        if not self.request.user.is_superuser:
            order_item = OrderItem.objects.filter(user=self.request.user)
            serializer = OrderItemSerializer(order_item, many=True)
            return Response(prepare_success_response(serializer.data), status=status.HTTP_200_OK)
        else:
            order_item = OrderItem.objects.all()
            serializer = OrderItemSerializer(order_item, many=True)
            return Response(prepare_success_response(serializer.data), status=status.HTTP_200_OK)


class CreateOrderView(generics.CreateAPIView):
    queryset = Order.objects.all()
    serializer_class = OrderSerializer

    def post(self, request, *args, **kwargs):
        try:
            serializer = OrderSerializer(data=request.data)
            if serializer.is_valid(raise_exception=True):
                serializer.save(customer=request.user)
                return Response(prepare_create_success_response(serializer.data), status=status.HTTP_201_CREATED)
            return Response(prepare_error_response(serializer.errors), status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            return Response(prepare_success_response(str(e)), status=status.HTTP_400_BAD_REQUEST)


class OrderListHistoryAPIView(generics.ListAPIView):
    queryset = Order.objects.all()
    serializer_class = OrderListDetailSerializer

    def list(self, request, *args, **kwargs):
        try:
            if self.request.user.is_superuser:
                order = Order.objects.all().order_by('id')
                serializer = OrderListDetailSerializer(order, many=True)
                return Response(prepare_success_response(serializer.data), status=status.HTTP_200_OK)
            else:
                order = Order.objects.filter(customer=self.request.user, ordered=True).order_by('-id')
                serializer = OrderListDetailSerializer(order, many=True)
                return Response(prepare_success_response(serializer.data), status=status.HTTP_200_OK)
        except Exception as e:
            return Response(prepare_success_response(str(e)), status=status.HTTP_400_BAD_REQUEST)


class OrderUpdateDetailsView(views.APIView):
    permission_classes = (permissions.IsAdminUser,)

    def get_object(self, order_ide):
        try:
            return Order.objects.get(order_ide=order_ide)
        except Order.DoesNotExist:
            return None

    def get(self, request, order_ide):
        try:
            details = self.get_object(order_ide)
            serializer = OrderListDetailSerializer(details)
            print(serializer.data)
            return Response(prepare_success_response(serializer.data), status=status.HTTP_200_OK)
        except Exception as e:
            return Response(prepare_success_response(str(e)), status=status.HTTP_400_BAD_REQUEST)

    def put(self, request, order_ide):
        try:
            order = self.get_object(order_ide)
            serializer = OrderSerializer(order, data=request.data)
            if serializer.is_valid(raise_exception=True):
                serializer.save(customer=request.user)
                return Response(prepare_create_success_response(serializer.data), status=status.HTTP_201_CREATED)
            return Response(prepare_error_response(serializer.errors), status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            return Response(prepare_success_response(str(e)), status=status.HTTP_400_BAD_REQUEST)



from enum import IntEnum


class TYPES(IntEnum):
    KG = 0
    PCS = 1
    BOX = 2

    @classmethod
    def select_types(cls):
        return [(key.value, key.name) for key in cls]


class STATUS(IntEnum):
    PENDING = 0
    CANCEL = 1
    FAILED = 2
    PROCESSING = 3
    DELIVERY = 4
    DONE = 5

    @classmethod
    def order_status(cls):
        return [(key.value, key.name) for key in cls]


class PAYMENT(IntEnum):
    CASH_ON_DELIVERY = 0
    BKASH = 1
    NOGOD = 2
    BANK = 3

    @classmethod
    def payment_choices(cls):
        return [(key.value, key.name) for key in cls]



from rest_framework import serializers

from commerce.models.product import Product
from commerce.models.category import Tag
from commerce.serializers.category_serializer import CategorySerializer, TagSerializer


class ProductSerializer(serializers.ModelSerializer):
    tags = TagSerializer(many=True)

    class Meta:
        model = Product
        read_only_fields = ('owner',)
        fields = (
            'id', 'item_name', 'owner', 'categories', 'tags', 'is_available', 'price', 'discount_price',
            'short_description', 'model_number', 'brand_name', 'serial_number', 'item_type', 'item_image',
            'galley_image', 'galley_image2', 'galley_image3', 'created_at', 'updated_at'

        )

    def to_representation(self, instance):
        response = super().to_representation(instance)
        response['categories'] = CategorySerializer(instance.categories).data
        return response

    def get_or_create_tag(self, tags):
        tag_ids = []
        for tag in tags:
            tag_instance, create = Tag.objects.get_or_create(pk=tag.get('id'), defaults=tag)
            tag_ids.append(tag_instance.pk)
        return tag_ids

    def create_or_update_tag(self, tags):
        tag_ids = []
        for tag in tags:
            tag_instance, create = Tag.objects.update_or_create(pk=tag.get('id'), defaults=tag)
            tag_ids.append(tag_instance.pk)
        return tag_ids

    def create(self, validated_data):
        tags = validated_data.pop('tags', [])
        product = Product.objects.create(**validated_data)
        product.tags.set(self.get_or_create_tag(tags))
        return product

    def update(self, instance, validated_data):
        tags = validated_data.pop('tags', [])
        instance.tags.set(self.create_or_update_tag(tags))
        return instance


class GenericProductSerializer(serializers.ModelSerializer):
    class Meta:
        model = Product
        depth = 2
        fields = '__all__'



from rest_framework import generics, permissions

from commerce.models.category import Category, Model, Brand
from commerce.serializers.category_serializer import CategorySerializer, ModelSerializer, BrandSerializer
from commerce.utils.permissions import ReadOnly


class CategoryCrateListView(generics.ListCreateAPIView):
    queryset = Category.objects.all()
    serializer_class = CategorySerializer
    permission_classes = (permissions.IsAdminUser | ReadOnly,)


class CategoryUpdateDeleteView(generics.UpdateAPIView, generics.DestroyAPIView):
    queryset = Category.objects.all()
    serializer_class = CategorySerializer
    permission_classes = (permissions.IsAdminUser | ReadOnly,)


class ModelAPIView(generics.ListCreateAPIView):
    queryset = Model.objects.all()
    serializer_class = ModelSerializer
    permission_classes = (permissions.IsAdminUser | ReadOnly,)


class ModelUpdateDeleteView(generics.UpdateAPIView, generics.DestroyAPIView):
    queryset = Model.objects.all()
    serializer_class = ModelSerializer


class BrandCrateListAPIView(generics.ListCreateAPIView):
    queryset = Brand.objects.all()
    serializer_class = BrandSerializer
    permission_classes = (permissions.IsAdminUser | ReadOnly,)


class BrandUpdateDeleteView(generics.UpdateAPIView, generics.DestroyAPIView):
    queryset = Brand.objects.all()
    serializer_class = BrandSerializer
    permission_classes = (permissions.IsAdminUser,)



from rest_framework import serializers

from commerce.models.order import Order, OrderItem
from commerce.serializers.product_serializer import ProductSerializer


class OrderItemSerializer(serializers.ModelSerializer):
    class Meta:
        read_only_fields = ('user',)
        model = OrderItem
        fields = (
            'id', 'user', 'ordered', 'item', 'quantity', 'get_total_price',
            'get_total_discounted_price', 'get_amount_saved', 'get_final_price',
            'created_at', 'updated_at',
        )

    def to_representation(self, instance):
        response = super().to_representation(instance)
        response['item'] = ProductSerializer(instance.item).data
        return response


class OrderSerializer(serializers.ModelSerializer):
    class Meta:
        model = Order
        read_only_fields = ('customer',)
        fields = (
            'id', 'customer', 'ref_code', 'items', 'ordered', 'billing_address',
            'being_delivered', 'received', 'status', 'payment', 'get_total',
            'order_ide', 'created_at', 'updated_at',
        )


class OrderListDetailSerializer(serializers.ModelSerializer):
    class Meta:
        model = Order
        depth = 2
        fields = (
            'id', 'customer', 'ref_code', 'items', 'ordered', 'billing_address',
            'being_delivered', 'received', 'status', 'payment', 'get_total',
            'order_ide', 'created_at', 'updated_at',
        )



from rest_framework import serializers

from commerce.models.category import Category, Tag, Model, Brand


class CategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Category
        fields = '__all__'


class TagSerializer(serializers.ModelSerializer):
    class Meta:
        model = Tag
        fields = (
            'id', 'name'
        )


class ModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = Model
        fields = '__all__'


class BrandSerializer(serializers.ModelSerializer):
    class Meta:
        model = Brand
        fields = '__all__'



from django.db import models
from commerce.models.core import CoreEntity


class Category(CoreEntity):
    name = models.CharField(max_length=70, unique=True)
    parent = models.ForeignKey('self', on_delete=models.CASCADE, blank=True, null=True, related_name='parentCategory')
    is_active = models.BooleanField(default=True)
    image = models.ImageField(upload_to='category/%y/%m', blank=True, null=True)

    def __str__(self):
        return self.name


class Model(CoreEntity):
    name = models.CharField(max_length=50, unique=True)
    is_active = models.BooleanField(default=True)

    def __str__(self):
        return self.name


class Brand(CoreEntity):
    name = models.CharField(max_length=50, unique=True)
    is_active = models.BooleanField(default=True)
    image = models.ImageField(upload_to='brand/%y/%m', blank=True, null=True)

    def __str__(self):
        return self.name


class Tag(CoreEntity):
    name = models.CharField(max_length=50)

    def __str__(self):
        return self.name



from django.urls import path

from commerce.views import category_view
from commerce.views import product_view
from commerce.views import order_view

urlpatterns = [
    # Model API endpoints
    path('model/', category_view.ModelAPIView.as_view(), name='model_api_view'),
    path('model/<pk>/', category_view.ModelUpdateDeleteView.as_view(), name='model_update_delete'),
    # Category API endpoints
    path('category/', category_view.CategoryCrateListView.as_view(), name='category_crate_list'),
    path('category/<pk>/', category_view.CategoryUpdateDeleteView.as_view(), name='category_update_delete'),
    # Brand API endpoints
    path('brand/', category_view.BrandCrateListAPIView.as_view(), name='brand_crate_list'),
    path('brand/<pk>/', category_view.BrandUpdateDeleteView.as_view(), name='brand_update_delete'),
    # product API endpoints
    path('products/', product_view.AllProductsView.as_view(), name='all_products'),
    path('product/', product_view.CreateListProductView.as_view(), name='product_crate_list'),
    path('product/<slug>/', product_view.ProductDetailsUpdateView.as_view(), name='product_update_details'),
    path('product-delete/<pk>/', product_view.ProductDeleteAPIView.as_view(), name='product_delete'),
    # Order Item API endpoints
    path('order-item/', order_view.CreateListOrderItemView.as_view(), name='order_item_list_create'),
    # Order API endpoints
    path('order/', order_view.CreateOrderView.as_view(), name='order_create'),
    path('orders/', order_view.OrderListHistoryAPIView.as_view(), name='order_list_history'),
    path('order/<str:order_ide>/', order_view.OrderUpdateDetailsView.as_view(), name='order_update_details'),
]
